/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
NX.define('Nexus.clm.SettingsView', {
    extend: 'Ext.Panel',

    requirejs: [
      'Nexus/siesta',
      'Nexus/clm'
    ],

    mixins: [
      'Nexus.LogAwareMixin'
    ],

    constructor: function (config) {
        var self = this;

        self.url = new Ext.form.TextField({
            name: 'url',
            fieldLabel: 'IQ Server URL',
            helpText: 'The address of your IQ Server',
            anchor : Sonatype.view.FIELD_OFFSET,
            allowBlank : false,
            itemCls : 'required-field',
            validator : function(v) {
              if (v === '' || v.match(/^(?:http|https):\/\//i))
              {
                return true;
              }
              else
              {
                return 'Protocol must be http:// or https://';
              }
            }
        });

        self.testConnectionStore = new Ext.data.JsonStore({
          id: 'clmTestConnection',
          fields: [{
            name: 'name',
            sortType: Ext.data.SortTypes.asUCString
          }],
          sortInfo: {
            field: 'name',
            direction: 'ASC'
          },
          data: []
        });

        self.testConnectionGrid = new Ext.grid.GridPanel({
          store: self.testConnectionStore,
          hidden: true,
          title: 'Available Applications',
          anchor: Sonatype.view.FIELD_OFFSET,
          height: 300,
          tools: [{
            type: 'close',
            handler: function(event, toolEl, panel) {
              panel.hide();
            }
          }],
          viewConfig: {
            autoFill: true,
            forceFit: true
          },
          colModel: new Ext.grid.ColumnModel({
            defaults: {
              sortable: true
            },
            columns: [{
              header: 'Application',
              dataIndex: 'name'
            }]
          })
        });
        
        self.settingsForm = new Ext.FormPanel({
          border : false,
          labelWidth : 100,
          layoutConfig : {
            labelSeparator : ''
          },
            items: [
                self.url,
                {
                  xtype: 'combo',
                  itemCls: 'required-field',
                  name: 'authenticationType',
                  fieldLabel: 'Authentication Method',
                  valueField: 'id',
                  displayField: 'name',
                  triggerAction: 'all',
                  emptyText: 'Select...',
                  selectOnFocus: true,
                  editable: false,
                  allowBlank: false,
                  store: [
                    ['USER', 'User Authentication'],
                    ['PKI', 'PKI Authentication']
                  ],
                  listeners: {
                    select: {
                      fn: function(combo) {
                        var newValue = combo.getValue();
                        var usernameField = this.settingsForm.find( 'name', 'username' )[0];
                        var passwordField = this.settingsForm.find( 'name', 'password' )[0];

                        if (newValue === 'USER') {
                          usernameField.enable();
                          usernameField.show();
                          passwordField.enable();
                          passwordField.show();

                          this.settingsForm.doLayout();
                        }
                        else {
                          usernameField.disable();
                          usernameField.hide();
                          passwordField.disable();
                          passwordField.hide();

                          this.settingsForm.doLayout();
                        }
                      },
                      scope: this
                    }
                  }
                },
                {
                  xtype: 'textfield',
                  itemCls: 'required-field',
                  name: 'username',
                  fieldLabel: 'Username',
                  helpText: 'User with access to the IQ Server.',
                  anchor: Sonatype.view.FIELD_OFFSET,
                  allowBlank: false,
                  hidden: true
                },
                {
                  xtype: 'textfield',
                  itemCls: 'required-field',
                  name: 'password',
                  inputType: 'password',
                  fieldLabel: 'Password',
                  helpText: 'Credentials for the IQ Server user.',
                  anchor: Sonatype.view.FIELD_OFFSET,
                  allowBlank: false,
                  hidden: true
                },
                {
                  xtype: 'numberfield',
                  name: 'timeout',
                  fieldLabel: 'Request Timeout',
                  helpText: 'Number of seconds to wait for a successful IQ Server connection before retrying. Defaults to 15 minutes.',
                  anchor : Sonatype.view.FIELD_OFFSET,
                  allowBlank : true
                },
              {
                xtype: 'textarea',
                fieldLabel: 'Properties',
                name: 'properties',
                anchor : Sonatype.view.FIELD_OFFSET,
                helpText: 'Additional properties to configure for the IQ Server',
                flex: 1
              }
            ],
            buttons : [
              {
                text : 'Save',
                tooltip : 'Save settings',
                formBind : true,
                scope : self,
                handler : self.saveSettings
              },
              { text : 'Reset',
                tooltip : 'Reset changes',
                formBind : true,
                scope : this,
                handler : self.loadSettings
              },
              { text : 'Test Connection',
                tooltip : 'Test connection and permissions.',
                formBind : true,
                scope : self,
                handler : self.testConnection
              }
            ]
        });

      self.disabledWarning = NX.create('Ext.Panel', {
        border : false,
        html : '<div class="nx-clm-disabled-warning">'
            +     '<div>'
            +       'IQ Server functionality is currently disabled. To enable it, use the Capabilities UI.'
            +     '</div>'
            + '</div>',
        hidden: true
      });

      Ext.apply(self, {
          cls: 'nx-clm-SettingsView',
          border: false,
          layout: 'vbox',
          layoutConfig: {
              align: 'stretch'
          },
          items: [
              {
                  xtype: 'fieldset',
                  title: 'Settings',
                  anchor : Sonatype.view.FIELDSET_OFFSET,
                  autoHeight : true,
                  items: [
                    {
                      border : false,
                      html : '<div class="nx-clm-doco">'
                            +   '<a href="http://links.sonatype.com/products/clm/nexus/home"  target="_blank">'
                            +   'Make Nexus Staging even more powerful with Nexus IQ Server'
                            +   ' </a>'
                            + '</div>'
                    },
                    self.disabledWarning,
                    self.settingsForm,
                    self.testConnectionGrid
                  ]
              }
          ]
      });

      self.constructor.superclass.constructor.apply(self, arguments);

      // Load settings after layout, has to be done post superclass.constructor()
      self.on('afterlayout', self.loadSettings, self, {
        single: true
      });

    },

    saveSettings : function() {
      var self = this,
          form, url;

      self.logDebug('IQ Server saving settings');

      form = self.settingsForm.getForm();

      if(!form.isValid())
      {
        return;
      }

      url = form.findField('url').getValue();

      Ext.Ajax.request({
        url : Nexus.clm.resourcePath,
        method : 'POST',
        scope : self,
        suppressStatus: 400,
        jsonData : form.getFieldValues(),

        success : function() {
          var nexusPanel = Sonatype.repoServer.RepoServer.nexusPanel;
          Nexus.clm.showMessage('Configuration saved');
          form.items.each(function(item){
            item.clearInvalid();
          });
          nexusPanel.get('st-enterprise-staging').remove(nexusPanel.clmMenu);
          Nexus.clm.serverUrl = url;
          nexusPanel.clmMenu = nexusPanel.add({
            enabled: true,
            sectionId: 'st-enterprise-staging',
            title: Nexus.clm.TITLE_DASHBOARD,
            href: encodeURI(Nexus.clm.serverUrl)
          });
          nexusPanel.doLayout();
        },

        failure: function(response, opts){
          if(response.siestaValidationError !== undefined)
          {
            Ext.each(response.siestaValidationError, function (error){
              form.findField(error.id).markInvalid(error.message);
            });
          }
        }
      });
    },

    loadSettings : function() {
      var self = this;

      self.logDebug('IQ Server loading settings');

      Ext.Ajax.request({
        url :  Nexus.clm.resourcePath,
        method : 'GET',
        scope : self,
        suppressStatus: 404,
        success : function(response) {
          var config = Ext.decode(response.responseText);
          self.settingsForm.getForm().setValues(config);
          if(config.enabled){
            self.disabledWarning.hide();
          }
          else {
            self.disabledWarning.show();
          }
          self.logDebug('IQ Server settings loaded');
        }
      });
    },

    testConnection: function() {
      var self = this, form, url;

      self.logDebug('Testing IQ Server connection.');

      form = self.settingsForm.getForm();

      if (!form.isValid()) {
        return;
      }

      Ext.Ajax.request({
        url: Nexus.clm.testConnectionPath,
        method: 'POST',
        scope: self,
        suppressStatus: 400,
        jsonData: form.getFieldValues(),
        success: function(response) {
          self.testConnectionGrid.show();
          self.testConnectionStore.loadData(JSON.parse(response.responseText));
        },
        failure: function(response, opts) {
          self.testConnectionGrid.hide();
          if (response.siestaValidationError !== undefined) {
            Ext.each(response.siestaValidationError, function(error) {
              form.findField(error.id).markInvalid(error.message);
            });
          }
        }
      });
    }
},

function () {
    var type = this;

    // Attach SettingsView to navigation panel if current user has appropriate permissions
    Sonatype.Events.addListener('nexusNavigationInit', function (nexusPanel) {
      var p = Sonatype.lib.Permissions;
      if (p.checkPermission('nexus:settings', p.READ)) {
        nexusPanel.add({
          enabled: true,
          sectionId: 'st-nexus-config',
          title: Nexus.clm.TITLE,
          tabId: Nexus.clm.TAB_ID,
          tabTitle: Nexus.clm.TAB_TITLE,
          tabCode: type
        });
      }
    });
});
